unit FrmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, Globals, Spin, ExtCtrls, ComCtrls, IniFiles;

{$WARN UNSAFE_TYPE OFF}

type
  TFormMain = class(TForm)
    PageControl: TPageControl;
    TabSheetWork: TTabSheet;
    TabSheetExplan: TTabSheet;
    OpenDialog: TOpenDialog;
    Panel1: TPanel;
    SpeedButtonSelectInputFiles: TSpeedButton;
    SpeedButtonDoIt: TSpeedButton;
    SpeedButtonExit: TSpeedButton;
    Label1: TLabel;
    EditTabsize: TEdit;
    SpinButtonTabSize: TSpinButton;
    CheckBoxUnTab: TCheckBox;
    CheckBoxUnWeb: TCheckBox;
    Panel2: TPanel;
    MemoFiles: TMemo;
    Memo1: TMemo;
    CheckBoxUnBlank: TCheckBox;
    Timer1: TTimer;
    procedure SpeedButtonExitClick(Sender: TObject);
    procedure SpeedButtonSelectInputFilesClick(Sender: TObject);
    procedure SpeedButtonDoItClick(Sender: TObject);
    procedure SpinButtonTabSizeUpClick(Sender: TObject);
    procedure SpinButtonTabSizeDownClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CheckBoxUnWebClick(Sender: TObject);
    procedure CheckBoxUnTabClick(Sender: TObject);
    procedure CheckBoxUnBlankClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
      FTabSize  : Integer;
      FFileList : TStringList;
      FUnTab    : Boolean;
      FUnWeb    : Boolean;
      FUnBlank  : Boolean;
      FInitDir  : String;
      FFiltInd  : Integer;
      FFstTm    : Boolean;
      FData     : TStringList;
    Procedure   SelectInputFiles;
    Procedure   ReadFile( Const aFileName: String);
    Procedure   ProcessUnWeb;
    Procedure   ProcessUnTab;
    Procedure   ProcessUnBlank;
    Procedure   WriteFile( Const aFileName: String);
    Procedure   ProcessInputFile( Const aFileName: String);
    Procedure   ProcessInputFiles;

    Procedure   LoadIniFile;
    Procedure   SaveIniFile;

    Procedure   SetTabSize( aValue: Integer);
    Procedure   SetUnTab  ( aValue: Boolean);
    Procedure   SetUnWeb  ( aValue: Boolean);
    Procedure   SetUnBlank( aValue: Boolean);

    Property    TabSize: Integer Read FTabSize Write SetTabSize;
    Property    UnTab  : Boolean Read FUnTab   Write SetUnTab;
    Property    UnWeb  : Boolean Read FUnWeb   Write SetUnWeb;
    Property    UnBlank: Boolean Read FUnBlank Write SetUnBlank;
    Property    InitDir: String  Read FInitDir Write FInitDir;
    Property    FiltInd: Integer Read FFiltInd Write FFiltInd;
  public
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

// User area


// ============================================================================

    Procedure   TFormMain.SelectInputFiles;
    Var
      i : Integer;
    Begin
      With OpenDialog Do
      Begin
        InitialDir  := InitDir;
        FilterIndex := FiltInd;
        If Execute
        Then Begin
          If Files.Count > 0
          Then InitDir := ExtractFilePath( Files[ 0]);
          FiltInd := FilterIndex;
          MemoFiles.Clear;
          FFileList.Clear;
          For i := 0 To Files.Count - 1 Do
          Begin
            MemoFiles.Lines.Add( Files[ i]);
            FFileList.Add( Files[ i]);
          End;
        End;
      End;
    End;

// ============================================================================

    Procedure   TFormMain.ReadFile( Const aFileName: String);
    Var
      InFile : File Of Char;
      S      : String;
      C      : Char;
    Begin
      AssignFile( InFile, aFileName);
      Reset( InFile);
      FData.Clear;
      S := '';
      Try
        While Not Eof( InFile) Do
        Begin
          Read( InFile, C);
          Case C Of
            ^M, ^J : Begin
                If S <> ''
                Then FData.Add( S);
                S := '';
              End;
            Else S := S + C;
          End;
        End;
        If S <> ''
        Then  FData.Add( S);
      Finally
        CloseFile( InFile);
      End;
    End;

// ============================================================================

    Procedure   TFormMain.ProcessUnWeb;

      Function ProcessLine( Const S: String): String;
      Const
        HexDigits = [ '0' .. '9', 'a' .. 'f', 'A' .. 'F'];
      Var
        i      : Integer;
        State  : ( stChar, stEq1, steq2);
        HexStr : String[ 2];
      Begin
        Result := '';
        State  := stChar;
        For i := 1 To Length( S) Do
        Begin
          Case State Of
            stChar : Begin
                Case S[ i] Of
                  '=' : Begin
                      State  := stEq1;
                      HexStr := '';
                    End;
                  Else Result := Result + S[ i];
                End;
              End;
            stEq1 : Begin
                If S[ i] In HexDigits
                Then Begin
                  HexStr := HexStr + S[ i];
                  State  := stEq2;
                End
                Else Begin
                  Result := Result + '=' + S[ i];
                  State  := stChar;
                End;
              End;
            stEq2 : Begin
                If S[ i] In HexDigits
                Then Begin
                  HexStr := HexStr + S[ i];
                  Result := Result + Char( StrToIntDef( Format( '$%s', [ HexStr]), $20));
                End
                Else Begin
                  Result := Result + '=' + HexStr[ 1] + S[ i];
                End;
                State := stChar;
              End;
          End;
        End;
        Case State Of
          stEq1 : Result := Result + '=';
          stEq2 : Result := Result + '=' + HexStr[ 1];
        End;
      End;

    Var
      i : Integer;
    Begin
      With FData Do
        For i := 0 To Count - 1 Do
          Strings[ i] := ProcessLine( Strings[ i])
    End;

// ============================================================================

    Procedure   TFormMain.ProcessUnTab;

      Function ProcessLine( Const S: String): String;
      Var
        i : Integer;
      Begin
        Result := '';
        For i := 1 To Length( S) Do
        Begin
          If S[ i] = ^I
          Then Begin
            Result := Result + ' ';
            While ( Length( Result) Mod TabSize) <> 0 Do
              Result := Result + ' ';
          End
          Else Result := Result + S[ i]
        End;
      End;

    Var
      i : Integer;
    Begin
      With FData Do
        For i := 0 To Count - 1 Do
          Strings[ i] := ProcessLine( Strings[ i])
    End;

// ============================================================================

    Procedure   TFormMain.ProcessUnBlank;

      Function ProcessLine( Const S: String): String;
      Var
        L : Integer;
      Begin
        Result := S;
        L := Length( S);
        While ( L > 0) And  ( Result[ L] In [ ' ', ^I]) Do
          Dec( L);
        SetLength( Result, L);
      End;

    Var
      i : Integer;
    Begin
      With FData Do
        For i := 0 To Count - 1 Do
          Strings[ i] := ProcessLine( Strings[ i])
    End;

// ============================================================================

    Procedure   TFormMain.WriteFile( Const aFileName: String);
    Begin
      FData.SaveToFile( aFileName);
    End;

// ============================================================================

    Procedure   TFormMain.ProcessInputFile( Const aFileName: String);
    Begin
      Caption := aFileName;
      ReadFile( aFileName);
      If UnWeb   Then ProcessUnWeb;
      If UnTab   Then ProcessUntab;
      If UnBlank Then ProcessUnBlank;
      WriteFile( aFileName);
    End;

// ============================================================================

    Procedure   TFormMain.ProcessInputFiles;
    Var
      i : Integer;
    Begin
      Try
      Timer1.Enabled := True;
        With FFileList Do
        Begin
          For i := 0 To Count - 1 Do
            ProcessInputFile( Strings[ i])
        End;
      Finally
        Timer1.Enabled := False;
        Caption := 'UN';
      End;
    End;

// ============================================================================

    Procedure   TFormMain.LoadIniFile;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          TabSize := ReadInteger( 'Settings', 'TabSize', TabSize);
          UnTab   := ReadBool   ( 'Settings', 'UnTab'  , UnTab  );
          UnWeb   := ReadBool   ( 'Settings', 'UnWeb'  , UnWeb  );
          UnBlank := ReadBool   ( 'Settings', 'UnBlank', UnBlank);
          InitDir := ReadString ( 'Settings', 'InitDir', InitDir);
          FiltInd := ReadInteger( 'Settings', 'FiltInd', FiltInd);
          FFstTm  := ReadBool   ( 'Settings', 'FstTm'  , True   );
        Finally
          Free;
        End;
    End;

// ============================================================================

    Procedure   TFormMain.SaveIniFile;
    Begin
      With TIniFile.Create( IniFileName) Do
        Try
          WriteInteger( 'Settings', 'TabSize', TabSize);
          WriteBool   ( 'Settings', 'UnTab'  , UnTab  );
          WriteBool   ( 'Settings', 'UnWeb'  , UnWeb  );
          WriteBool   ( 'Settings', 'UnBlank', UnBlank);
          WriteString ( 'Settings', 'InitDir', InitDir);
          WriteInteger( 'Settings', 'FiltInd', FiltInd);
          WriteBool   ( 'Settings', 'FstTm'  , False  );
        Finally
          Free;
        End;
    End;

// ============================================================================

    Procedure   TFormMain.SetTabSize( aValue: Integer);
    Begin
      If aValue <> FTabSize
      Then Begin
        FTabSize := aValue;
        EditTabSize.Text := IntToStr( FTabSize);
      End;
    End;

// ============================================================================

    Procedure   TFormMain.SetUnTab( aValue: Boolean);
    Begin
      If aValue <> FUnTab
      Then Begin
        FUnTab := aValue;
        CheckBoxUnTab.Checked := aValue;
      End;
    End;

// ============================================================================

    Procedure   TFormMain.SetUnWeb( aValue: Boolean);
    Begin
      If aValue <> FUnWeb
      Then Begin
        FUnWeb := aValue;
        CheckBoxUnWeb.Checked := aValue;
      End;
    End;

// ============================================================================

    Procedure   TFormMain.SetUnBlank( aValue: Boolean);
    Begin
      If aValue <> FUnBlank
      Then Begin
        FUnBlank := aValue;
        CheckBoxUnBlank.Checked := aValue;
      End;
    End;

// ============================================================================

// Delphi area

procedure TFormMain.SpeedButtonExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.SpeedButtonSelectInputFilesClick(Sender: TObject);
begin
  SelectInputFiles;
end;

procedure TFormMain.SpeedButtonDoItClick(Sender: TObject);
begin
  ProcessInputFiles;
end;

procedure TFormMain.SpinButtonTabSizeUpClick(Sender: TObject);
begin
  If TabSize < 32
  Then TabSize := TabSize + 1;
end;

procedure TFormMain.SpinButtonTabSizeDownClick(Sender: TObject);
begin
  If TabSize > 1
  Then TabSize := TabSize - 1;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  FTabSize := 0;
  TabSize  := 8;
  FFileList := TStringList.Create;
  FData     := TStringList.Create;
  LoadIniFile;
  If FFstTm
  Then PageControl.ActivePage := TabSheetExplan
  Else PageControl.ActivePage := TabSheetWork;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveIniFile;
  FreeAndNil( FFileList);
  FreeAndNil( FData);
end;

procedure TFormMain.CheckBoxUnWebClick(Sender: TObject);
begin
  UnWeb := CheckBoxUnWeb.Checked;
end;

procedure TFormMain.CheckBoxUnTabClick(Sender: TObject);
begin
  UnTab := CheckBoxUnTab.Checked;
end;

procedure TFormMain.CheckBoxUnBlankClick(Sender: TObject);
begin
  UnBlank := CheckBoxUnBlank.Checked;
end;

procedure TFormMain.Timer1Timer(Sender: TObject);
begin
  Application.ProcessMessages;
end;

end.
