unit Globals;

interface

Uses

  SysUtils, Windows, Messages, StdCtrls, Forms, Dialogs;


{$DEFINE DEBUG}
{$UNDEF DEBUG}

{$WARN UNSAFE_CODE OFF}

Const

  WM_USERNAMECHANGED = WM_USER + 0;

// Our name and path stuff

  Function ApplicationPath    : String;
  Function ApplicationPathURL : String;
  Function ApplicationFileName: String;
  Function IniFileName        : String;
  Function LogFileName        : String;
  Function FileNameToURL( Const aFileName: String): String;

Const

  strZeroTime  = '0:00';
  strBlankTime = '';
  strDotTime   = '..';

  Function  MinutesToStr    ( aValue      : Word  ): String;
  Function  MinutesToStrZero( aValue      : Word  ): String;    
  Function  MinutesToStrDots( aValue      : Word  ): String;
  Function  StrToMinutes    ( Const aValue: String): Integer;
  Function  WeekDayToStr    ( aDay        : Word  ): String;
  Function  ByteToHex       ( aValue: Cardinal): String;
  Function  WordToHex       ( aValue: Cardinal): String;
  Function  CellToHex       ( aValue: Cardinal): String;
  Function  LongToHex       ( aValue: Cardinal): String;
  Function  HexToByte       ( Const aValue: String): Cardinal;
  Function  HexToWord       ( Const aValue: String): Cardinal;
  Function  HexToCell       ( Const aValue: String): Cardinal;
  Function  HexToLong       ( Const aValue: String): Cardinal;
  Function  ByteToAscii     ( aValue: Byte): Char;

// Debug support

{$IFDEF DEBUG}
  Procedure ClearDebugTiming;
  Procedure WriteDebugStr( Const aString: String);
{$ENDIF}

// TNumber

Type

  TNumber = Class( TObject)
      Number : Integer;
    Constructor Create( aNumber : Integer);
  End;


// System stuff

  Function  KeyboardRepeatTime: Integer;
  Function  KeyBoardInitialDelay: Integer;
  Function  Maximum(A, B: Integer): Integer;
  Function  Minimum(A, B: Integer): Integer;
  Function  CleanFileName( Const aFileName : String): String;
  // Returns filename only, path and extension are stripped off.


// Miscellaneous

  Procedure NotImplemented( Const aMsg: String); 
  Procedure Noop;



implementation


// Our name and path stuff

Var

  strApplicationPath     : String;
  strApplicationPathURL  : String;
  strApplicationFileName : String;
  strIniFileName         : String;
  strLogFileName         : String;

  Function ApplicationPath: String;
  Begin
    Result := strApplicationPath;
  End;

  Function ApplicationPathURL : String;
  Begin
    Result := strApplicationPathURL;
  End;

  Function ApplicationFileName: String;
  Begin
    Result := strApplicationFileName;
  End;

  Function IniFileName: String;
  Begin
    Result := strIniFileName;
  End;

  Function LogFileName: String;
  Begin
    Result := strLogFileName;
  End;

  Function FileNameToURL( Const aFileName: String): String;
  Var
    i : Integer;
  Begin
    Result := 'file:///';
    For i := 1 To Length( aFileName) Do
      Case aFileName[ i] Of
        ':' : Result := Result + '|';
        '\' : Result := Result + '/';
        Else  Result := Result + aFileName[ i];
      End;
  End;

// Date time stuff

Const

  WeekDays : Array [ 0 .. 6] Of String = (
    'maandag',
    'dinsdag',
    'woensdag',
    'donderdag',
    'vrijdag',
    'zaterdag',
    'zondag'
  );

  Function  MinutesToStr( aValue: Word): String;
  Begin
    If aValue = 0
    Then Result := strBlankTime
    Else Result := Format( '%d:%.2d', [ aValue Div 60, aValue Mod 60]);
  End;

  Function MinutesToStrZero( aValue: Word): String;
  Begin
    If aValue = 0
    Then Result := strZeroTime
    Else Result := Format( '%d:%.2d', [ aValue Div 60, aValue Mod 60]);
  End;

  Function  MinutesToStrDots( aValue: Word): String;
  Begin
    If aValue = 0
    Then Result := strDotTime
    Else Result := Format( '%d:%.2d', [ aValue Div 60, aValue Mod 60]);
  End;

  Function  WeekDayToStr( aDay: Word): String;
  Begin
    If aDay In [ 0 .. 6]
    Then Result := WeekDays[ aDay]
    Else Result := 'ongeldige dag';
  End;

  Function  StrToMinutes( Const aValue: String): Integer;
  Var
    p      : Integer;
    S1, S2 : String;
    V1, V2 : Integer;
  Begin
    Result := 0;
    p := Pos( ':', aValue);
    If p > 0
    Then Begin
      S1 := Copy( aValue, 1, p -1);
      S2 := Copy( aValue, p + 1, 2);
      V1 := StrToIntDef( S1, 0);
      V2 := StrToIntDef( S2, 0);
      Result := V1 * 60 + V2;
    End;
  End;

  Function  ByteToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.2x', [ aValue And $00ff]);
  End;

  Function  WordToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.4x', [ aValue And $ffff]);
  End;

  Function  CellToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.2x.%0.4x', [ aValue And $00ff0000 Shr 16, aValue And $ffff]);
  End;

  Function  LongToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.4x.%0.4x', [ aValue And $ffff0000 Shr 16, aValue And $ffff]);
  End;

  Function  HexToByte( Const aValue: String): Cardinal;
  Begin
    Result := StrToInt( Format( '$%s', [ aValue]));
  End;

  Function  HexToWord( Const aValue: String): Cardinal;
  Begin
    Result := StrToInt( Format( '$%s', [ aValue]));
  End;

  Function  HexToCell( Const aValue: String): Cardinal;
  Var
    S : String;
    p : Integer;
  Begin
    S := aValue;
    p := Pos( '.', S);
    While p > 0 Do
    Begin
      S := Copy( S, 1, p - 1) + Copy( S, p + 1, Length( S));
      p := Pos( '.', S);
    End;
    Result := StrToInt( Format( '$%s', [ S]));
  End;

  Function  HexToLong( Const aValue: String): Cardinal;
  Var
    S : String;
    p : Integer;
  Begin
    S := aValue;
    p := Pos( '.', S);
    While p > 0 Do
    Begin
      S := Copy( S, 1, p - 1) + Copy( S, p + 1, Length( S));
      p := Pos( '.', S);
    End;
    Result := StrToInt( Format( '$%s', [ S]));
  End;

  Function  ByteToAscii( aValue: Byte): Char;
  Begin
    If ( aValue < 32) Or ( aValue >= 127)
    Then Result := '.'
    Else Result := Char( aValue);
  End;

// Debug support

{$IFDEF DEBUG}
Var

  DebugFile : Text;
  DebugName : String;
  DebugTime : Integer;

  Procedure ClearDebugTiming;
  Begin
    DebugTime := GetTickCount;
  End;

  Function FormatDebugTime( aTime: Integer): String;
  Var
    day, hr, min, sec, ms : Integer;
  Begin
    ms    := aTime Mod 1000;
    aTime := aTime Div 1000;
    sec   := aTime Mod   60;
    aTime := aTime Div   60;
    min   := aTime Mod   60;
    aTime := aTime Div   60;
    hr    := aTime Mod   24;
    aTime := aTime Div   24;
    day   := aTime;
    Result := Format( '%.2d:%.2d:%.2d.%.', [ day, hr, min, sec, ms]);
  End;

  Procedure WriteDebugStr( Const aString: String);
  Begin
    Append( DebugFile);
    WriteLn(
      DebugFile,
      Format(
        '%s - %s', [ FormatDebugTime( GetTickCount - DebugTime), aString ]
      )
    );
    Close( DebugFile);
  End;
{$ENDIF}


  Procedure InitModule;
  Var
    S : String;
    p : Integer;
  Begin
    S := Application.ExeName;
    strApplicationFileName := ExtractFileName( S);
    strApplicationPath     := ExtractFilePath( S);
    strApplicationPathURL  := FileNameToURL( strApplicationPath);
    p := Pos( '.', strApplicationFileName);
    If p = 0
    Then Begin
      strIniFileName := strApplicationPath + strApplicationFileName + '.INI';
      strLogFileName := strApplicationPath + strApplicationFileName + '.LOG';
    End
    Else Begin
      strIniFileName := strApplicationPath + Copy( strApplicationFileName, 0, p - 1) + '.INI';
      strLogFileName := strApplicationPath + Copy( strApplicationFileName, 0, p - 1) + '.LOG';
    End;

{$IFDEF DEBUG}
    DebugName := strApplicationPath + 'debug.txt';
    Assign( DebugFile, DebugName);
    Rewrite( DebugFile);
    Close( DebugFile);
    ClearDebugTiming;
    WriteDebugStr( 'Program start');
{$ENDIF}

  End;

  
// TNumber support

{ ========
  TNumber = Class( TObject)
      Number : Integer;
}

    Constructor TNumber.Create( aNumber : Integer);
    Begin
      Inherited Create;
      Number := aNumber;
    End;

// System stuff

  Function KeyboardRepeatTime: Integer;
  Begin
    If Not SystemParametersInfo( SPI_GETKEYBOARDSPEED, 0, @ Result, 0)
    Then Result := 2;
    If Result = 0
    Then Result := 1000
    Else Result := Round( 1000 / Result);
  End;

  Function KeyBoardInitialDelay: Integer;
  Begin
    // @@@@ nog aanpassen & uitzoeken
    Result := 250;
  End;

  Function Maximum(A, B: Integer): Integer;
  Begin
    If A > B
    Then Result := A
    Else Result := B;
  End;

  Function Minimum(A, B: Integer): Integer;
  Begin
    If A < B
    Then Result := A
    Else Result := B;
  End;

  Function  CleanFileName( Const aFileName : String): String;
  Var
    i : Integer;
  Begin
    Result := ExtractFileName( aFileName);
    i := Pos( '.', Result);
    If i > 0
    Then Result := Copy( Result, 1, i - 1);
  End;



// Miscellaneous

  Procedure NotImplemented( Const aMsg: String);
  Begin
    MessageDlg(
      Format(
        'Not implemented : %s',
        [ aMsg]
      ),
      mtInformation,
      [ mbOk, mbCancel],
      0
    );
  End;

  Procedure Noop;
  Begin
  End;


// Initialisation:
//   get version info from executable
//   setup debug stuff
//   etc.

Initialization

  InitModule;

Finalization

end.


